﻿using System;
using System.Diagnostics;
using System.Linq;
using System.Reflection;
using System.Threading;
using System.Web;
using EPiCurrency = Mediachase.Commerce.Currency;
using Mediachase.Commerce.Catalog;
using Mediachase.Commerce.Catalog.Managers;
using Mediachase.Commerce.Core;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.Orders.Dto;
using Mediachase.Commerce.Orders.Managers;
using Mediachase.Commerce.Website;
using Mediachase.Commerce.Website.Helpers;
using Klarna.Api;
using System.Collections.Generic;
using KlarnaApiAddress = Klarna.Api.Address;

namespace EPiServer.Business.Commerce.Payment.Klarna
{
    /// <summary>
    /// This class intent to support both Klarna Invoice payment and Klarna Part Payment gateway.
    /// </summary>
    internal class KlarnaHelper
    {
        internal const string Swe = "SWE";
        internal const string Fin = "FIN";
        internal const string Nor = "NOR";
        internal const string Dnk = "DNK";
        internal const string Deu = "DEU";
        internal const string Nld = "NLD";
        internal const string Aut = "AUT";

        internal const string SwedishPclasssValues = "SwedishPclasssValues";
        internal const string FinnishPclasssValues = "FinnishPclasssValues";
        internal const string GermanPclasssValues = "GermanPclasssValues";
        internal const string DutchPclasssValues = "DutchPclasssValues";
        internal const string DanishPclasssValues = "DanishPclasssValues";
        internal const string NorwegianPclasssValues = "NorwegianPclasssValues";
        internal const string AustriaPclasssValues = "AustriaPclasssValues";

        internal const string TestFlag = "TestFlag";
        internal const string MerchantId = "MerchantId";
        internal const string SharedSecret = "SharedSecret";
        internal const string EnableCustomerEnterAddress = "EnableCustomerEnterAddress";
        internal const string NeedUpdatePclass = "NeedUpdatePclass";
        internal const string KlarnaPartPayment = "KlarnaPartPayment";
        internal const string KlarnaInvoice = "KlarnaInvoice";

        internal static readonly string PcUriPartPayment = HttpContext.Current.Server.MapPath("~/Templates/Sample/Units/CartCheckout/KlarnaPartPayment/ClientResources/storage.xml");
        internal static readonly string PcUriInvoice = HttpContext.Current.Server.MapPath("~/Templates/Sample/Units/CartCheckout/KlarnaInvoice/ClientResources/storage.xml");

        internal enum KlarnaPayment
        {
            InvoicePayment,
            PartPayment
        }

        internal const string ShippingFeeCodeTemplate = "Shipping-fee-{0}";

        /// <summary>
        /// Gets the klarna country enum by country code
        /// </summary>
        /// <param name="countryCode">The country code.</param>
        /// <returns>The klarna country enum value</returns>
        internal static Country.Code GetKlarnaCountry(string countryCode)
        {
            switch (countryCode)
            {
                case Swe:
                    return Country.Code.SE;
                case Fin:
                    return Country.Code.FI;
                case Nor:
                    return Country.Code.NO;
                case Dnk:
                    return Country.Code.DK;
                case Deu:
                    return Country.Code.DE;
                case Nld:
                    return Country.Code.NL;
                case Aut:
                    return Country.Code.AT;
                default:
                    return Country.Code.SE;
            }
        }

        /// <summary>
        /// Gets the Klarna country encoding by country code
        /// </summary>
        /// <param name="countryCode">The country code.</param>
        /// <returns>The klarna encoding enum</returns>
        internal static Encoding GetKlarnaEncodingByCountry(string countryCode)
        {
            switch (countryCode)
            {
                case Swe:
                    return Encoding.Sweden;
                case Fin:
                    return Encoding.Finland;
                case Nor:
                    return Encoding.Norway;
                case Dnk:
                    return Encoding.Denmark;
                case Deu:
                    return Encoding.Germany;
                case Nld:
                    return Encoding.Netherlands;
                case Aut:
                    return Encoding.Austria;
                default:
                    return Encoding.Sweden;
            }
        }

        /// <summary>
        /// Gets the currency code by country.
        /// </summary>
        /// <param name="countryCode">The country code.</param>
        /// <returns>The currency code string</returns>
        internal static EPiCurrency GetCurrencyByCountry(string countryCode)
        {
            switch (countryCode)
            {
                case Swe:
                    return EPiCurrency.SEK;
                case Fin:
                    return EPiCurrency.EUR;
                case Nor:
                    return EPiCurrency.NOK;
                case Dnk:
                    return EPiCurrency.DKK;
                case Deu:
                    return EPiCurrency.EUR;
                case Nld:
                    return EPiCurrency.EUR;
                case Aut:
                    return EPiCurrency.EUR;
                default:
                    return EPiCurrency.SEK;
            }
        }

        /// <summary>
        /// Gets the store name by country.
        /// </summary>
        /// <param name="countryCode">The country code.</param>
        /// <returns>A store name based on country</returns>
        internal static string GetStoreNameByCountry(string countryCode)
        {
            switch (countryCode)
            {
                case Swe:
                    return SwedishPclasssValues;
                case Fin:
                    return FinnishPclasssValues;
                case Nor:
                    return NorwegianPclasssValues;
                case Dnk:
                    return DanishPclasssValues;
                case Deu:
                    return GermanPclasssValues;
                case Nld:
                    return DutchPclasssValues;
                case Aut:
                    return AustriaPclasssValues;
                default:
                    return string.Empty;
            }
        }

        /// <summary>
        /// Gets the klarna base config. EID, SharedSecret, servermode
        /// </summary>
        /// <param name="klarnaPayment">The klarna payment type </param>
        /// <returns>A klarnaConfig object which store configured parameter</returns>
        internal static IConfiguration GetKlarnaBaseConfig(KlarnaPayment klarnaPayment)
        {
            IConfiguration klarnaConfig = new global::Klarna.Api.Configuration(Country.Code.SE, Language.Code.SV, Currency.Code.SEK, Encoding.Sweden);

            // MerchantId parameter
            var merchantIdparameter = GetParameterByName(MerchantId, klarnaPayment);
            if (merchantIdparameter != null)
            {
                klarnaConfig.Eid = int.Parse(merchantIdparameter.Value);
            }

            // SharedSecret parameter
            var sharedSecretparameter = GetParameterByName(SharedSecret, klarnaPayment);
            if (sharedSecretparameter != null)
            {
                klarnaConfig.Secret = sharedSecretparameter.Value;
            }

            var serverMode = GetParameterByName(TestFlag, klarnaPayment);
            if (serverMode != null)
            {
                bool isTestMode = false;
                bool.TryParse(serverMode.Value, out isTestMode);
                klarnaConfig.IsLiveMode = !isTestMode;
            }

            return klarnaConfig;
        }

        /// <summary>
        /// Determines whether [is allow different address].
        /// </summary>
        /// <param name="klarnaPayment">The klarna payment type </param>
        /// <returns>
        /// 	<c>true</c> if [is allow different address]; otherwise, <c>false</c>.
        /// </returns>
        internal static bool IsAllowDifferentAddress(KlarnaPayment klarnaPayment)
        {
            bool result = false;

            // Allow different address parameter
            var allowDifferentAddressParameter = GetParameterByName(EnableCustomerEnterAddress, klarnaPayment);
            if (allowDifferentAddressParameter != null)
            {
                bool.TryParse(allowDifferentAddressParameter.Value, out result);
            }

            return result;
        }


        /// <summary>
        /// Determines whether [is update pclass].
        /// </summary>
        /// <param name="klarnaPayment">The klarna payment type </param>
        /// <returns>
        /// 	<c>true</c> if [is update pclass]; otherwise, <c>false</c>.
        /// </returns>
        internal static bool IsUpdatePclass(KlarnaPayment klarnaPayment)
        {
            bool isUpdate = false;

            // NeedUpdatePclassParameter parameter
            var needUpdatePclass = GetParameterByName(NeedUpdatePclass, klarnaPayment);
            if (needUpdatePclass != null)
            {
                isUpdate = needUpdatePclass.Value.ToUpper().Equals("YES");
                if (isUpdate)
                {
                    // Set parameter to "no"
                    needUpdatePclass.Value = "no";

                    var paymentMethodDto = PaymentManager.GetPaymentMethodBySystemName(KlarnaPartPayment, SiteContext.Current.LanguageName);
                    if (paymentMethodDto == null)
                    {
                        string method = MethodInfo.GetCurrentMethod().Name;
                        ErrorManager.GenerateError(string.Format("Can't get payment method Dto. Error in {0}", method));
                    }
                    // Save the setting
                    PaymentManager.SavePayment(paymentMethodDto);
                }
            }

            return isUpdate;
        }

        /// <summary>
        /// Updates the klarna config by country. Including Country code, language code, currency code, encoding
        /// </summary>
        /// <param name="countryCode">The country code.</param>
        /// <param name="klarnaConfig">The klarna config.</param>
        internal static void UpdateKlarnaConfigByCountry(string countryCode, IConfiguration klarnaConfig)
        {
            switch (countryCode)
            {
                case Swe:
                    klarnaConfig.Country = Country.Code.SE;
                    klarnaConfig.Language = Language.Code.SV;
                    klarnaConfig.Currency = Currency.Code.SEK;
                    klarnaConfig.Encoding = Encoding.Sweden;
                    break;
                case Fin:
                    klarnaConfig.Country = Country.Code.FI;
                    klarnaConfig.Language = Language.Code.FI;
                    klarnaConfig.Currency = Currency.Code.EUR;
                    klarnaConfig.Encoding = Encoding.Finland;
                    break;
                case Nor:
                    klarnaConfig.Country = Country.Code.NO;
                    klarnaConfig.Language = Language.Code.NB;
                    klarnaConfig.Currency = Currency.Code.NOK;
                    klarnaConfig.Encoding = Encoding.Norway;
                    break;
                case Dnk:
                    klarnaConfig.Country = Country.Code.DK;
                    klarnaConfig.Language = Language.Code.DA;
                    klarnaConfig.Currency = Currency.Code.DKK;
                    klarnaConfig.Encoding = Encoding.Denmark;
                    break;
                case Deu:
                    klarnaConfig.Country = Country.Code.DE;
                    klarnaConfig.Language = Language.Code.DE;
                    klarnaConfig.Currency = Currency.Code.EUR;
                    klarnaConfig.Encoding = Encoding.Germany;
                    break;
                case Nld:
                    klarnaConfig.Country = Country.Code.NL;
                    klarnaConfig.Language = Language.Code.NL;
                    klarnaConfig.Currency = Currency.Code.EUR;
                    klarnaConfig.Encoding = Encoding.Netherlands;
                    break;
                case Aut:
                    klarnaConfig.Country = Country.Code.AT;
                    klarnaConfig.Language = Language.Code.DE;
                    klarnaConfig.Currency = Currency.Code.EUR;
                    klarnaConfig.Encoding = Encoding.Austria;
                    break;
                default:
                    break;
            }
        }

        /// <summary>
        /// Gets the country code from billing address.
        /// </summary>
        /// <returns>A Country code</returns>
        internal static string GetCountryCode()
        {
            var currentCart = new CartHelper(Cart.DefaultName);
            var billingAddressId = currentCart.OrderForm.BillingAddressId;
            var address = currentCart.FindAddressByName(billingAddressId);

            return (address == null) ? Swe : address.CountryCode;
        }

        /// <summary>
        /// Configs the Klarna object
        /// </summary>
        /// <param name="klarna">The klarna.</param>
        /// <param name="klarnaPayment">The klarna payment type </param>
        /// <returns>A Klarna object which has been configured</returns>
        internal static Api ConfigKlarna(KlarnaPayment klarnaPayment)
        {
            IConfiguration klarnaConfig = GetKlarnaBaseConfig(klarnaPayment);
            UpdateKlarnaConfigByCountry(GetCountryCode(), klarnaConfig);

            var klarna = new Api(klarnaConfig);
            if (klarnaPayment == KlarnaPayment.PartPayment)
            {
                klarna.PClassStorageUri = PcUriPartPayment;
            }
            else
            {
                klarna.PClassStorageUri = PcUriInvoice;
            }

            //It's important that we get the end customers IP attached to the Klarna Object
            klarna.ClientIp = HttpContext.Current.Request.UserHostAddress;

            return klarna;
        }

        /// <summary>
        /// Gets the name of the parameter by.
        /// </summary>
        /// <param name="parameterName">Name of the parameter.</param>
        /// <param name="klarnaPayment">The klarna payment type </param>
        /// <returns>The parameter</returns>
        private static PaymentMethodDto.PaymentMethodParameterRow GetParameterByName(string parameterName, KlarnaPayment klarnaPayment)
        {
            string systemName = klarnaPayment == KlarnaPayment.InvoicePayment ? KlarnaInvoice : KlarnaPartPayment;
            var paymentMethodDto = PaymentManager.GetPaymentMethodBySystemName(systemName, SiteContext.Current.LanguageName);
            if (paymentMethodDto == null)
            {
                string method = MethodInfo.GetCurrentMethod().Name;
                ErrorManager.GenerateError(string.Format("Can't get payment method Dto. Error in {0}", method));
            }

            var rowArray = (PaymentMethodDto.PaymentMethodParameterRow[])paymentMethodDto.PaymentMethodParameter.Select(string.Format("Parameter = '{0}'", parameterName));
            if ((rowArray != null) && (rowArray.Length > 0))
            {
                return rowArray[0];
            }

            return null;
        }

        /// <summary>
        /// Sets the klarna address.
        /// </summary>
        /// <param name="klarnaClient">The klarna client.</param>
        /// <param name="billingAddress">The billing address.</param>
        /// <param name="shippingAddress">The shipping address.</param>
        /// <param name="houseNumber">The house number.</param>
        /// <param name="houseExtension">The house extension.</param>
        internal static void SetKlarnaAddress(Api klarnaClient, OrderAddress billingAddress, OrderAddress shippingAddress, string houseNumber, string houseExtension)
        {
            // Make a Klarna address
            klarnaClient.BillingAddress = new KlarnaApiAddress(){
                Email = billingAddress.Email,
                PhoneNumber = billingAddress.EveningPhoneNumber,
                CellPhoneNumber = billingAddress.DaytimePhoneNumber,
                FirstName = billingAddress.FirstName,
                LastName = billingAddress.LastName,
                Street = string.Empty,
                CareOf = billingAddress.Line1,
                ZipCode = billingAddress.PostalCode,
                City = billingAddress.City,
                Country = GetKlarnaCountry(billingAddress.CountryCode),
                HouseNumber = houseNumber,
                HouseExtension = houseExtension,
                CompanyName = string.Empty
            };

            klarnaClient.ShippingAddress = new KlarnaApiAddress()
            {
                Email = shippingAddress.Email,
                PhoneNumber = shippingAddress.EveningPhoneNumber,
                CellPhoneNumber = shippingAddress.DaytimePhoneNumber,
                FirstName = shippingAddress.FirstName,
                LastName = shippingAddress.LastName,
                Street = string.Empty,
                CareOf = shippingAddress.Line1,
                ZipCode = shippingAddress.PostalCode,
                City = shippingAddress.City,
                Country = GetKlarnaCountry(shippingAddress.CountryCode),
                HouseNumber = houseNumber,
                HouseExtension = houseExtension,
                CompanyName = string.Empty
            };
        }

        /// <summary>
        /// Gets the item tax in percent.
        /// </summary>
        /// <param name="item">The item.</param>
        /// <param name="addressByName">Name of the address by.</param>
        /// <returns>Return the tax in percent</returns>
        private static decimal GetItemSaleTax(LineItem item, OrderAddress addressByName)
        {
            var itemTax = 0M;
            // Get the catalogEntryDto
            var catalogEntryDto = CatalogContext.Current.GetCatalogEntryDto(item.CatalogEntryId, new CatalogEntryResponseGroup(CatalogEntryResponseGroup.ResponseGroup.Variations));
            if (catalogEntryDto.CatalogEntry.Count > 0)
            {
                // Get the variationrows
                var variationRows = catalogEntryDto.CatalogEntry[0].GetVariationRows();
                if (variationRows.Length > 0)
                {
                    // Get TaxCategory
                    var taxCategoryNameById = CatalogTaxManager.GetTaxCategoryNameById(variationRows[0].TaxCategoryId);
                    var valueArray = OrderContext.Current.GetTaxes(Guid.Empty, taxCategoryNameById, Thread.CurrentThread.CurrentCulture.Name, addressByName.CountryCode, addressByName.State, addressByName.PostalCode, addressByName.RegionCode, string.Empty, addressByName.City);
                    if (valueArray.Length > 0)
                    {
                        itemTax = (decimal)valueArray.Where(v => v.TaxType == TaxType.SalesTax).Sum(v => v.Percentage);
                    }
                }
            }
            return itemTax;
        }

        /// <summary>
        /// Determines whether [is currency correct] [the specified currency].
        /// </summary>
        /// <param name="currency">The currency.</param>
        /// <returns>
        /// 	<c>true</c> if [is currency correct] [the specified currency]; otherwise, <c>false</c>.
        /// </returns>
        internal static bool IsCurrencyCorrect(EPiCurrency currency)
        {
            return SiteContext.Current.Currency.Equals(currency);
        }

        /// <summary>
        /// Gets the currency name by code.
        /// </summary>
        /// <param name="currencyCode">The currency.</param>
        /// <returns>A string currency name</returns>
        internal static EPiCurrency GetCurrencyNameByCode(EPiCurrency currency)
        {
            switch (currency.CurrencyCode)
            {
                case "SEK":
                    return "Swedish krona";
                case "EUR":
                    return "Euro";
                case "NOK":
                    return "Norwegian krone";
                case "DKK":
                    return "Danish krone";
                default:
                    return currency.CurrencyCode;
            }
        }

        /// <summary>
        /// Determines whether the current call stack is start from specific source type to destination source type.
        /// </summary>
        /// <param name="destinaion">The destinaion type.</param>
        /// <param name="source">The source type.</param>
        /// <returns>
        /// 	<c>true</c> if the current call stack is start from specific source type to destination source type; otherwise, <c>false</c>.
        /// </returns>
        internal static bool IsCallingFrom(Type destinaion, Type source)
        {
            var stackTrace = new StackTrace();
            StackFrame[] stackFrames = stackTrace.GetFrames();

            if (stackFrames == null)
            {
                return false;
            }

            var stackFrameAtDestination = stackFrames.FirstOrDefault(sf => sf.GetMethod().DeclaringType.UnderlyingSystemType == destinaion);
            var stackFrameAtSource = stackFrames.LastOrDefault(sf => sf.GetMethod().DeclaringType.UnderlyingSystemType == source);

            if (stackFrameAtDestination == null || stackFrameAtSource == null)
            {
                return false;
            }

            return stackFrames.ToList().IndexOf(stackFrameAtDestination) < stackFrames.ToList().IndexOf(stackFrameAtSource);
        }

        /// <summary>
        /// Updates the invoice number in Klarna sytem based on given invoice number to new invoice number which is get from purchase order.
        /// </summary>
        /// <param name="klarnaClient">The proxy to communicate with Klarna server.</param>
        /// <param name="processedForm">The processed form, which contains the tracking number to update</param>
        /// <param name="oldInvoiceNumber">The old invoice number.</param>
        /// <returns>True if success</returns>
        internal static bool UpdateKlarnaInvoiceNumber(Api klarnaClient, OrderForm processedForm, string oldInvoiceNumber)
        {
            if (processedForm == null || processedForm.Parent == null)
            {
                return false;
            }
            var purchaseOrder = processedForm.Parent as PurchaseOrder;
            if (purchaseOrder == null)
            {
                return false;
            }

            var newInvoiceNumber = purchaseOrder.TrackingNumber;
            if (!String.IsNullOrEmpty(oldInvoiceNumber) && !String.IsNullOrEmpty(newInvoiceNumber))
            {
                try
                {
                    // Update the orderNumber for the invoice
                    klarnaClient.UpdateOrderNumber(oldInvoiceNumber, newInvoiceNumber);
                }
                catch (KlarnaException kex)
                {
                    ErrorManager.GenerateError(kex.Message);
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// Validates the currency.
        /// </summary>
        /// <param name="countryCode">The country code to validate.</param>
        /// <returns>True if the currency is supported in the country of billing address.</returns>
        internal static bool ValidateCurrency(string countryCode)
        {
            switch (countryCode)
            {
                case Swe:
                case Fin:
                case Nor:
                case Dnk:
                case Deu:
                case Nld:
                case Aut:
                    // Klarna only support for the countries above.
                    if (!IsCurrencyCorrect(GetCurrencyByCountry(countryCode)))
                    {
                        return false;
                    }
                    break;
            }

            return true;
        }

        /// <summary>
        /// Updates the order address.
        /// </summary>
        /// <param name="form">The form contains order address to update.</param>
        /// <param name="klarnaAddress">The klarna address.</param>
        internal static void UpdateOrderAddress(OrderForm form, Address klarnaAddress)
        {
            if (form == null)
            {
                throw new ArgumentNullException("form");
            }

            if (klarnaAddress == null)
            {
                throw new ArgumentNullException("klarnaAddress");
            }

            foreach (var address in form.Parent.OrderAddresses.Cast<OrderAddress>().Where(address => address.Name == form.BillingAddressId))
            {
                address.LastName = klarnaAddress.LastName;
                address.Line1 = klarnaAddress.Street;
                address.City = klarnaAddress.City;
                address.PostalCode = klarnaAddress.Zip;
                form.Parent.AcceptChanges();
                break;
            }
        }

        /// <summary>
        /// Get line items of current shipment
        /// </summary>
        /// <param name="payment">The payment.</param>
        /// <param name="shipment">The shipment.</param>
        /// <returns>The Article number list</returns>
        internal static List<ArticleNumber> ArticleNoCollection(Orders.KlarnaPayment payment, Shipment shipment)
        {
            var articleNoList = new List<ArticleNumber>();

            if (shipment != null)
            {
                int shipmentIndex = shipment.Parent.Shipments.IndexOf(shipment) + 1;

                foreach (var lineItem in Shipment.GetShipmentLineItems(shipment))
                {
                    if (lineItem != null)
                    {
                        var quantity = Shipment.GetLineItemQuantity(shipment, lineItem.LineItemId);
                        articleNoList.Add(new ArticleNumber((int)quantity, lineItem.CatalogEntryId));
                    }
                }
            }

            return articleNoList;
        }

        /// <summary>
        /// Generates the order number by "PO" + cart.OrderGroupId + Random(100-999)
        /// </summary>
        /// <param name="cart">The OrderGroupId, used to take from from Cart.OrderGroupId</param>
        /// <returns>Random string to be OrderNumber</returns>
        public static string GenerateOrderNumber(int OrderGroupId)
        {
            string str = new Random().Next(100, 999).ToString();
            return string.Format("PO{0}{1}", OrderGroupId, str);
        }

        /// <summary>
        /// Add an Order Form to Klarna Client
        /// </summary>
        /// <param name="API">The Klarna client api</param>
        /// <param name="OrderForm">The order form</param>
        /// <returns></returns>
        internal static void AddOrderForm(Api klarnaClient, OrderForm orderForm)
        {
            var shipments = orderForm.Shipments;
            var itemList = new List<Article>();
            var shippingList = new List<Article>();

            foreach (Shipment shipment in shipments)
            {
                var shippingAddress = orderForm.Parent.OrderAddresses.Cast<OrderAddress>().FirstOrDefault(a => string.Equals(a.Name, shipment.ShippingAddressId, StringComparison.Ordinal));
                int shipmentIndex = shipments.IndexOf(shipment) + 1;

                foreach (var lineItem in Shipment.GetShipmentLineItems(shipment))
                {
                    // Calculate the tax for a line item
                    var taxRate = GetItemSaleTax(lineItem, shippingAddress);
                    var quantity = Shipment.GetLineItemQuantity(shipment, lineItem.LineItemId);
                    var itemPriceExcTax = lineItem.PlacedPrice - ((lineItem.OrderLevelDiscountAmount + lineItem.LineItemDiscountAmount) / quantity);
                    decimal itemTax = taxRate > 0 ? (itemPriceExcTax * taxRate / 100) : 0;

                    itemList.Add(new Article((int)lineItem.Quantity, lineItem.CatalogEntryId, lineItem.DisplayName, (double)(itemPriceExcTax + itemTax), (double)taxRate, 0, GoodsFlags.IncVAT));
                }

                var discountShippingFee = shipment.ShipmentTotal - shipment.ShippingDiscountAmount;
                // calculate shipping tax in percentage
                var shippingTaxInPercent = discountShippingFee != 0 ? (double)Math.Floor((shipment.ShippingTax / discountShippingFee) * 100) : 0;
                
                // add shipping cost for each shipment, as a line item
                shippingList.Add(new Article(1, string.Format(ShippingFeeCodeTemplate, shipmentIndex), "Shipment fee", (double)(discountShippingFee + shipment.ShippingTax), shippingTaxInPercent, 0, GoodsFlags.IncVAT));
            }

            // group item by item number, to sum all quantity. So each line item will be displayed as one row.
            var articleList = (from item in itemList
                        group item by item.Number into g
                        select new Article()
                        {
                            Number = g.First().Number,
                            Title = g.First().Title,
                            Flags = g.First().Flags,
                            Quantity = g.Sum(i => i.Quantity),

                            Price = g.Sum(i => i.Price) / g.Count(), // get average value
                            VAT = g.Sum(i => i.VAT) / g.Count(), // get average value
                            Discount = g.Sum(i => i.Discount) / g.Count() // get average value
                        })
                        .Concat(shippingList);
            foreach (var article in articleList)
            {
                klarnaClient.AddArticle(article.Quantity, article.Number, article.Title, article.Price, article.VAT, article.Discount, article.Flags);
            }
        }

        /// <summary>
        /// Private class, represents an article in Klarna system.
        /// </summary>
        private class Article
        {
            public int Quantity { get; set; }
            public string Number { get; set; }
            public string Title { get; set; }
            public double Price { get; set; }
            public double VAT { get; set; }
            public double Discount { get; set; }
            public GoodsFlags Flags { get; set; }

            public Article(int quantity, string number, string title, double price, double vat, double discount, GoodsFlags flags)
            {
                Quantity = quantity;
                Number = number;
                Title = title;
                Price = price;
                VAT = vat;
                Discount = discount;
                Flags = flags;
            }

            public Article()
            { }
        }
    }
}
